﻿Public Class Form1

    ' Demonstrate inconsistency between SelectionStart and Text
    Private Sub UltraFormattedTextEditor1_Click(sender As Object, e As System.EventArgs) Handles UltraFormattedTextEditor1.EditStateChanged

        Dim iCurrentPosition As Integer = UltraFormattedTextEditor1.EditInfo.SelectionStart
        UltraLabel1.Text = ("Text around cursor: " & UltraFormattedTextEditor1.Text.Substring(Math.Max(iCurrentPosition - 3, 0), Math.Min(iCurrentPosition, 3)) & _
            " | " & UltraFormattedTextEditor1.Text.Substring(iCurrentPosition, Math.Min(3, UltraFormattedTextEditor1.Text.Length - iCurrentPosition))).Replace(vbLf, "#")

    End Sub

    ' Demonstrate bug with BeforeToolDropdown firing before SelectionStart updates
    Private Sub UltraToolbarsManager1_BeforeToolDropdown(sender As Object, e As Infragistics.Win.UltraWinToolbars.BeforeToolDropdownEventArgs) Handles UltraToolbarsManager1.BeforeToolDropdown
        UltraToolbarsManager1.Tools(1).SharedProps.Caption = "Cursor Position: " & UltraFormattedTextEditor1.EditInfo.SelectionStart
    End Sub


#Region "Drag/Drop"

    Private iPreviousSelectionLength As Integer = 0
    Private iPreviousSelectionStart As Integer = 0
    Private iHighlightStart As Integer = 0
    Private iHighlightLength As Integer = 0
    Private bAllowDragDrop As Boolean = False
    Private bResetPrevious As Boolean = False


    ' Record where the selection was previously, so we can tell if we click upon a selection
    Private Sub OOTextbox_EditStateChanged(sender As Object, e As Infragistics.Win.FormattedLinkLabel.EditStateChangedEventArgs) Handles UltraFormattedTextEditor1.EditStateChanged

        If bResetPrevious OrElse UltraFormattedTextEditor1.EditInfo.SelectionLength > 0 Then
            iPreviousSelectionLength = UltraFormattedTextEditor1.EditInfo.SelectionLength
            iPreviousSelectionStart = UltraFormattedTextEditor1.EditInfo.SelectionStart
            bResetPrevious = False
        End If
        If UltraFormattedTextEditor1.EditInfo.SelectionLength = 0 Then
            bResetPrevious = True
        End If

    End Sub


    ' Create a fake selection in place of an existing selection if the user clicks on it
    Private Sub OOTextbox_MouseDown(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles UltraFormattedTextEditor1.MouseDown

        bAllowDragDrop = False
        If e.Button = Windows.Forms.MouseButtons.Left AndAlso iPreviousSelectionLength > 0 Then ' SelectionLength > 0 Then
            Dim iMousePosition As Integer = UltraFormattedTextEditor1.EditInfo.GetCaretPositionFromMouseLocation(e.Location)

            If iMousePosition >= iPreviousSelectionStart AndAlso iMousePosition <= iPreviousSelectionStart + iPreviousSelectionLength Then

                iHighlightStart = iPreviousSelectionStart
                iHighlightLength = iPreviousSelectionLength

                With UltraFormattedTextEditor1.EditInfo
                    .SelectionStart = iHighlightStart
                    .SelectionLength = iHighlightLength
                    .ApplyStyle("Background-color:#" & Hex(SystemColors.Highlight.ToArgb) & ";color:#" & Hex(SystemColors.HighlightText.ToArgb) & ";", False)
                End With

                bAllowDragDrop = True
                iPreviousSelectionStart = 0
                iPreviousSelectionLength = 0
            End If
        End If

    End Sub


    ' Start the Drag/Drop when the user moves the mouse having clicked on a selection
    Private Sub OOTextbox_MouseMove(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles UltraFormattedTextEditor1.MouseMove

        If bAllowDragDrop AndAlso e.Button = Windows.Forms.MouseButtons.Left AndAlso iHighlightLength > 0 Then
            Dim iMousePosition As Integer = UltraFormattedTextEditor1.EditInfo.GetCaretPositionFromMouseLocation(e.Location)
            bAllowDragDrop = False

            If iMousePosition >= iHighlightStart AndAlso iMousePosition <= iHighlightStart + iHighlightLength Then
                ' Yuck, have to convert Carriage Returns because of inconsistency between SelectionStart and Text (Infragistics Bug)
                DoDragDrop(UltraFormattedTextEditor1.Text.Replace(vbCrLf, vbCr).Substring(iHighlightStart, iHighlightLength), DragDropEffects.Move Or DragDropEffects.Copy)
            End If
        End If

    End Sub


    ' Move the cursor to where the mouse is, so we can see where we would be dropping
    Private Sub rtxtSource_DragOver(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles UltraFormattedTextEditor1.DragOver

        Try
            If e.Data.GetDataPresent("System.String") Then

                ' Set cursor to where we're pointing
                Dim iNewIndex As Integer = UltraFormattedTextEditor1.EditInfo.GetCaretPositionFromMouseLocation(UltraFormattedTextEditor1.PointToClient(MousePosition))

                ' Move the cursor to the current mouse position
                UltraFormattedTextEditor1.EditInfo.SelectionStart = iNewIndex
                UltraFormattedTextEditor1.EditInfo.SelectionLength = 0                

                ' Don't allow dropping onto our existing selection
                If iHighlightLength > 0 AndAlso iNewIndex >= iHighlightStart AndAlso iNewIndex <= iHighlightStart + iHighlightLength Then
                    e.Effect = DragDropEffects.None
                Else
                    ' Move or Copy depending whether we have the Control key depressed
                    If (e.KeyState And 8) = 8 Then ' Ctrl key
                        e.Effect = DragDropEffects.Copy
                    Else
                        e.Effect = DragDropEffects.Move
                    End If
                End If

            End If
        Catch
        End Try

    End Sub


    ' Copy or Move text on Drag/Drop
    Private Sub rtxtSource_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles UltraFormattedTextEditor1.DragDrop

        Try
            If e.Data.GetDataPresent("System.String") Then
                With e.Data.GetData("System.String")
                    Dim iSelTextStart As Integer = UltraFormattedTextEditor1.EditInfo.SelectionStart

                    ' Insert the new value, and select it
                    UltraFormattedTextEditor1.EditInfo.InsertValue(.ToString)
                    UltraFormattedTextEditor1.EditInfo.SelectionLength = .ToString.Length

                    ' If it was a Move drag/drop, remove the previous text
                    If (e.Effect And DragDropEffects.Move) = DragDropEffects.Move Then
                        ' Adjust selection depending whether we dropped before or after old selection
                        If iSelTextStart > iHighlightStart Then
                            iSelTextStart -= iHighlightLength
                        Else
                            iHighlightStart += iHighlightLength
                        End If
                        UltraFormattedTextEditor1.EditInfo.SelectionStart = iHighlightStart
                        UltraFormattedTextEditor1.EditInfo.SelectionLength = iHighlightLength
                        UltraFormattedTextEditor1.EditInfo.Cut()
                    End If
                End With
            End If

        Catch
        End Try

    End Sub


    ' Clear any fake highlights on Mouse Up
    Private Sub OOTextbox_MouseUp(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles UltraFormattedTextEditor1.MouseUp
        If iHighlightLength > 0 Then
            iHighlightStart = 0
            iHighlightLength = 0
            UltraFormattedTextEditor1.Text = UltraFormattedTextEditor1.Text
        End If
    End Sub

#End Region

End Class
